SystemOrganization addCategory: #'Seaside-Adaptors-Swazoo'!

!WAPackage class methodsFor: '*seaside-adaptors-swazoo' stamp: 'lr 12/18/2008 20:25'!
seasideAdaptorsSwazoo
	^ self new
		name: 'Seaside-Adaptors-Swazoo';
		description: 'Makes Seaside run on the Swazoo Web server (LGPL).';
		addDependency: 'Seaside-Squeak-Core';
		addDependency: 'Seaside-Swazoo';
		yourself! !

WABaseServerAdaptor subclass: #WASwazooAdaptor
	instanceVariableNames: 'site'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Swazoo'!

!WASwazooAdaptor class methodsFor: 'actions' stamp: 'pmm 11/14/2008 17:04'!
resetAll
	"removes all Seaside sites, run this if you run into trouble creating servers"
	| server toRemove |
	server := SwazooServer singleton.
	toRemove := server allSites select: [ :each |
		each isKindOf: WAPlugableSite ].
	toRemove do: [ :each |
		server removeSite: each ]! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:41'!
basicStart
	site port: self port.
	site start! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:44'!
basicStop
	site stop! !

!WASwazooAdaptor methodsFor: 'private-converting' stamp: 'pmm 11/6/2008 14:53'!
convertCookiesFrom: aWAResonse to: aSwazooResponse
	| cookies setCookieField setCookie2Field |
	cookies := aWAResonse cookies.
	cookies isEmpty ifTrue: [ ^self ].
	
	setCookieField := WorkingHTTPSetCookieField new.
	setCookie2Field := WorkingHTTPSetCookie2Field new.

	cookies do: [ :each |
		setCookieField addCookie: each oldNetscapeString.
		setCookie2Field addCookie: each rfc2965String ].
	
	aSwazooResponse headers addField: setCookieField.
	aSwazooResponse headers addField: setCookie2Field! !

!WASwazooAdaptor methodsFor: 'private-converting' stamp: 'pmm 11/6/2008 14:49'!
convertHeader: aSwazooRequest
	| swazooFields fields |
	swazooFields := aSwazooRequest headers fields.
	fields := Dictionary new: swazooFields size.
	swazooFields keysAndValuesDo: [ :key :field |
		| value |
		value := String streamContents: [ :stream |
			field valuesAsStringOn: stream ].
		fields at: key asLowercase put: value ].
	^fields! !

!WASwazooAdaptor methodsFor: 'private-converting' stamp: 'lr 2/7/2009 12:38'!
convertResponse: aSeasideResponse
	| swazooResponse |
	swazooResponse := HTTPResponse new.
	
	aSeasideResponse headers do: [ :each |
		| field fieldClass |
		fieldClass := HeaderField classForFieldName: each key.
		field := fieldClass newForFieldName: each key withValueFrom: each value.
		swazooResponse headers addField: field ].
	self convertCookiesFrom: aSeasideResponse to: swazooResponse.
	
	swazooResponse code: aSeasideResponse status asNumber.
	swazooResponse entity: (self contentsOfResponse: aSeasideResponse).
	swazooResponse contentType: aSeasideResponse contentType seasideString.
	^swazooResponse! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:42'!
cookiesFrom: aSwazooRequest
	^WARequestCookie fromString: aSwazooRequest cookie! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'lr 2/7/2009 12:02'!
getFieldsOf: aSwazooRequest into: aDictionaryMerger
	aSwazooRequest queries do: [ :each |
		aDictionaryMerger at: each key put: (self url decode: each value) ]! !

!WASwazooAdaptor methodsFor: 'initialization' stamp: 'lr 1/10/2009 12:49'!
initialize
	super initialize.
	site := WAPlugableSite
		named: (WAKeyGenerator current keyOfLength: 32)
		on: self.
	site ip: '*'.
	site host: '*'.
	SwazooServer singleton addSite: site! !

!WASwazooAdaptor methodsFor: 'private-response creation' stamp: 'pmm 1/10/2009 17:06'!
internalErrorResponseFor: anError
	^HTTPResponse internalServerError
		contentType: 'text/plain';
		entity: 'Internal Error';
		yourself! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'pmm 11/8/2008 12:49'!
isRunning
	^site isServing! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'pmm 11/8/2008 12:51'!
isStarting
	^false! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'pmm 11/8/2008 12:50'!
isStopped
	^site isServing not! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'pmm 11/8/2008 12:50'!
isStopping
	^false! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/27/2008 07:53'!
methodNameOf: aNativeRequest
	^aNativeRequest methodName! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'lr 2/7/2009 12:02'!
postFieldsOf: aNativeRequest into: aDictionaryMerger
	"break encapsulation"
	aNativeRequest ensureFullRead.
	aNativeRequest postData keysAndValuesDo: [ :key :value |
		| previousData data |
		data := value filename isEmptyOrNil
			ifTrue: [ self url decode: value value ]
			ifFalse: [
				WAFile new
					fileName: (self url decode: value filename);
					contentType: value contentType seasideMimeType;
					contents: value value;
					yourself ].
		aDictionaryMerger at: key put: data ]! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:57'!
remoteAddressOf: aSwazooRequest
	"converts a Squeak address (ByteArray) to an IPv4 String"
	| remoteAddress |
	remoteAddress := aSwazooRequest peer.
	remoteAddress isString ifTrue: [
		"so apparently someone has a new VM with IPv6 support"
		^remoteAddress ].
	
	^String streamContents: [ :stream |
		remoteAddress
			do: [ :each | stream print: each ]
			separatedBy: [ stream nextPut: $. ] ]! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 1/10/2009 17:07'!
requestHandlingAborted
	^ (HttpResponse fromString: 'Request handling aborted; reload to retry')
		contentType: 'text/plain';
		yourself! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:46'!
shutDown
	SwazooServer stop! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/15/2008 10:45'!
startUp
	SwazooServer start! !

!WASwazooAdaptor methodsFor: 'accessing' stamp: 'pmm 11/8/2008 12:34'!
statusString
	^self isRunning
		ifTrue: [ 'running' ]
		ifFalse: [ 'stopped' ]! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/27/2008 22:30'!
urlFrom: aSwazooRequest
	^((WAUrl absolute: aSwazooRequest uri printString)
		decodedWith: self codec) withoutParameters! !
